/*************************************************** 
  This is an example for our Adafruit 16-channel PWM & Servo driver
  Servo test - this will drive 8 servos, one after the other on the
  first 8 pins of the PCA9685

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/products/815
  
  These drivers use I2C to communicate, 2 pins are required to  
  interface.

  Adafruit invests time and resources providing this open source code, 
  please support Adafruit and open-source hardware by purchasing 
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.  
  BSD license, all text above must be included in any redistribution
 ****************************************************/

#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();
// you can also call it with a different address you want
//Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x41);
// you can also call it with a different address and I2C interface
//Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x40, Wire);

// Depending on your servo make, the pulse width min and max may vary, you 
// want these to be as small/large as possible without hitting the hard stop
// for max range. You'll have to tweak them as necessary to match the servos you
// have!
#define fingerSERVOMIN  75 // This is the 'minimum' pulse length count (out of 4096)
#define fingerSERVOMAX  520 // This is the 'maximum' pulse length count (out of 4096)
#define defaultSERVOMIN  150 // This is the 'minimum' pulse length count (out of 4096)
#define defaultSERVOMAX  600 // This is the 'maximum' pulse length count (out of 4096)
#define USMIN  600 // This is the rounded 'minimum' microsecond length based on the minimum pulse of 150
#define USMAX  2400 // This is the rounded 'maximum' microsecond length based on the maximum pulse of 600
#define SERVO_FREQ 50 // Analog servos run at ~50 Hz updates

/////////////////////////////////////////////////////////////////////////
#define interruptNumber 0 //if we use pin 2 for the external interrupt, we need to attach  interrupt number 0
                          //pin 3 would require interrupt number 1
                          
#define SwitchPin 2    //connect switch output(ON) to D2 (and off pin to GND)
#define LED 13                  //we use the built in LED to show a switch event


volatile byte SwitchEventFlag;//all variables that are changed within an ISR need to be dimensioned as 'volatile'
                                //this variable will be used to tell the main loop that an interrupt event occurred.
//////////////////////////////////////////////////////////////////////////


void setup() {
  Serial.begin(9600);
  
  pinMode(SwitchPin, INPUT_PULLUP);//define the interrupt pin as an input
  attachInterrupt(interruptNumber,InterruptServiceRoutine, CHANGE);//CHANGE defines that the ISR is called when the voltage on the pin changes in either direction
              //we are using CHANGE and not LOW to 'debounce' the pad. Using LOW would result in a continuous reading of the touch event until your finger
              //is removed from the pad

  
  pwm.begin();
  /*
   * In theory the internal oscillator (clock) is 25MHz but it really isn't
   * that precise. You can 'calibrate' this by tweaking this number until
   * you get the PWM update frequency you're expecting!
   * The int.osc. for the PCA9685 chip is a range between about 23-27MHz and
   * is used for calculating things like writeMicroseconds()
   * Analog servos run at ~50 Hz updates, It is importaint to use an
   * oscilloscope in setting the int.osc frequency for the I2C PCA9685 chip.
   * 1) Attach the oscilloscope to one of the PWM signal pins and ground on
   *    the I2C PCA9685 chip you are setting the value for.
   * 2) Adjust setOscillatorFrequency() until the PWM update frequency is the
   *    expected value (50Hz for most ESCs)
   * Setting the value here is specific to each individual I2C PCA9685 chip and
   * affects the calculations for the PWM update frequency. 
   * Failure to correctly set the int.osc value will cause unexpected PWM results
   */
  pwm.setOscillatorFrequency(27000000);
  pwm.setPWMFreq(SERVO_FREQ);  // Analog servos run at ~50 Hz updates

  delay(10);

  
  //start all servos in box closed mode
  pwm.setPWM(0, 0, CanAngleToPulse(20) );
  delay(50);
  pwm.setPWM(1, 0, sprayAngleToPulse(0) );
  delay(50);
  pwm.setPWM(2, 0, fingerAngleToPulse(50) );
  delay(50);

  delay(200);
}



void loop() {


//  //Can housing rotation servo
//  //10 degrees = retracted
//  //95 degrees = vertical
//  pwm.setPWM(0, 0, CanAngleToPulse(10) );
//    delay(4000);
//
//  pwm.setPWM(0, 0, CanAngleToPulse(95) );
//    delay(4000);



//  //Sprayer Servo
//  //0 degrees = not spraying
//  //130 degrees = spraying
//  pwm.setPWM(1, 0, sprayAngleToPulse(0) );
//    delay(4000);
//
//  pwm.setPWM(1, 0, sprayAngleToPulse(130) );
//    delay(500); 
//    
//  pwm.setPWM(1, 0, sprayAngleToPulse(0) );
//    delay(4000);
  


//  // //finger servo
//  //40 degrees = retracted
//  //108 degrees = flip switch
//  pwm.setPWM(2, 0, fingerAngleToPulse(108) );
//      delay(500);
//  pwm.setPWM(2, 0, fingerAngleToPulse(40) );
//      delay(4000);

///////////////////////////////////////////

///////////////
handleSerial();

/////////////////////////////////////////


if (SwitchEventFlag==1){
  
    delay(2000);
    
//operating sequence description
// 1. rotate can up
// 2. squeeze spray 
// 3. release spray
// 4. rotate can down
// 5. finger flip switch
// 6. finger retract

 //operating sequence implementation
  pwm.setPWM(0, 0, CanAngleToPulse(103) ); // 1. rotate can up
      delay(2000);
  pwm.setPWM(1, 0, sprayAngleToPulse(135) ); // 2. squeeze spray
      delay(350);
  pwm.setPWM(1, 0, sprayAngleToPulse(0) ); // 3. release spray
      delay(2000);
  pwm.setPWM(0, 0, CanAngleToPulse(20) ); // 4. rotate can down
      delay(2000);
  pwm.setPWM(2, 0, fingerAngleToPulse(108) ); // 5. finger flip switch
      delay(500);
  pwm.setPWM(2, 0, fingerAngleToPulse(50) ); // 6. finger retract
      delay(2000);

  SwitchEventFlag=0;//reset the flag for the next interrupt

}
}

int fingerAngleToPulse(int ang){
  int pulse = map(ang, 0, 180, fingerSERVOMIN, fingerSERVOMAX);
//  Serial.print("Angle: ");Serial.print(ang);
//  Serial.print("Pulse: ");Serial.print(pulse);
  return pulse;

}

int CanAngleToPulse(int ang){
  int pulse = map(ang, 0, 180, defaultSERVOMIN, defaultSERVOMAX);
//  Serial.print("Angle: ");Serial.print(ang);
//  Serial.print("Pulse: ");Serial.print(pulse);
  return pulse;

}

int sprayAngleToPulse(int ang){
  int pulse = map(ang, 0, 180, defaultSERVOMIN, defaultSERVOMAX);
//  Serial.print("Angle: ");Serial.print(ang);
//  Serial.print("Pulse: ");Serial.print(pulse);
  return pulse;

}



void InterruptServiceRoutine()//ISRs need to be void and have no parameters.
{
  
if(digitalRead(SwitchPin)==HIGH)  //when the switch is off the output is HIGH
  {
    digitalWrite(LED, LOW);             //LED is off
  }
   
  if(digitalRead(SwitchPin)==LOW) //flipping the switch causes the output to go LOW
  {
    digitalWrite(LED, HIGH);//turn the LED on
    SwitchEventFlag=1;//set the flag to tell the main loop that a touch event occurred.
  }
  
}




// stuff (more specifically, this is stuff for manually operating the controls through serial monitor commands for controls demonstration purposes.)
void handleSerial() {
 while (Serial.available() > 0) {
   char incomingCharacter = Serial.read();
   switch (incomingCharacter) {
     case '1':
      Serial.println("1. rotate can up");
      pwm.setPWM(0, 0, CanAngleToPulse(103) ); // 1. rotate can up
      delay(2000);
      break;
 
     case '2':
      Serial.println("2. squeeze spray");
      pwm.setPWM(1, 0, sprayAngleToPulse(135) ); // 2. squeeze spray
      delay(350);
      break;

     case '3':
      Serial.println("3. release spray");
      pwm.setPWM(1, 0, sprayAngleToPulse(0) ); // 3. release spray
      delay(2000);
      break;

     case '4':
      Serial.println("4. rotate can down");
      pwm.setPWM(0, 0, CanAngleToPulse(20) ); // 4. rotate can down
      delay(2000);
      break;

     case '5':
      Serial.println("5. finger flip switch");
      pwm.setPWM(2, 0, fingerAngleToPulse(108) ); // 5. finger flip switch
      delay(500);
      break;

     case '6':
      Serial.println("6. finger retract");
      pwm.setPWM(2, 0, fingerAngleToPulse(50) ); // 6. finger retract
      delay(2000);
      break;

     case '7':
      Serial.println("Full Sequence");
       //operating sequence implementation
      pwm.setPWM(0, 0, CanAngleToPulse(103) ); // 1. rotate can up
      delay(2000);
      pwm.setPWM(1, 0, sprayAngleToPulse(135) ); // 2. squeeze spray
      delay(350);
      pwm.setPWM(1, 0, sprayAngleToPulse(0) ); // 3. release spray
      delay(2000);
      pwm.setPWM(0, 0, CanAngleToPulse(20) ); // 4. rotate can down
      delay(2000);
      pwm.setPWM(2, 0, fingerAngleToPulse(108) ); // 5. finger flip switch
      delay(500);
      pwm.setPWM(2, 0, fingerAngleToPulse(50) ); // 6. finger retract
      delay(2000);
      break;
    }
 }
}
